@@ -1,5 +1,52 @@
 context("Labelled")
 
+# var_label --------------------------------------------------------------
+
+test_that("var_label works properly", {
+  x <- 1:3
+  var_label(x) <- "value"
+  expect_equal(attr(x, "label"), "value")
+  expect_equal(var_label(x), "value")
+  var_label(x) <- NULL
+  expect_null(attr(x, "label"))
+  expect_null(var_label(x))
+})
+
+test_that("var_label works on data.frame", {
+  df <- data.frame(x = 1:3, y = c("a", "b", "c"))
+  var_label(df$x) <- "var x"
+  expect_equal(var_label(df$x), "var x")
+  expect_equal(var_label(df), list(x = "var x", y = NULL))
+  var_label(df) <- list(y = "YY", x = "XX")
+  expect_equal(var_label(df), list(x = "XX", y = "YY"))
+  var_label(df) <- NULL
+  expect_equal(var_label(df), list(x = NULL, y = NULL))
+  var_label(df) <- c("var1", "var2")
+  expect_equal(var_label(df), list(x = "var1", y = "var2"))
+  df <- set_variable_labels(df, x = "XX", .labels = "other")
+  expect_equal(var_label(df), list(x = "XX", y = "other"))
+  df <- set_variable_labels(df, .labels = c("var1", "var2"))
+  expect_equal(var_label(df), list(x = "var1", y = "var2"))
+})
+
+
+test_that("var_label produce appropriate errors", {
+  df <- data.frame(x = 1:3, y = c("a", "b", "c"))
+  expect_error(var_label(df) <- c("var1", "var2", "var3"))
+  expect_error(var_label(df) <- list(x = "xx", z = "zz"))
+})
+
+test_that("var_label preserved data.frame type", {
+  tb <- dplyr::tibble(x = 1:3, y = c("a", "b", "c"))
+  before <- class(tb)
+  var_label(tb$x) <- "var x"
+  var_label(tb) <- list(y = "YY", x = "XX")
+  after <- class(tb)
+  expect_equal(before, after)
+})
+
+# labelled --------------------------------------------------------------
+
 test_that("labelled return an object of class haven_labelled",{
   x <- labelled(c(1,2,3), c(yes = 1, maybe = 2, no = 3))
   expect_that(is.labelled(x), is_true())
@@ -21,7 +68,7 @@ test_that("labels must have names", {
 })
 
 
-# var_labels and var_label ------------------------------------------------
+# val_labels and val_label ------------------------------------------------
 
 test_that("val_labels preserves variable label", {
   x <- 1:3
@@ -61,12 +108,14 @@ test_that("val_labels and val_label preserves spss missing values", {
   expect_equal(attr(x, "labels", exact = TRUE), c(yes = 1, no = 3, maybe = 2))
 })
 
-test_that("value labels can't be removed if missing values are defined", {
+test_that("value labels can be removed if missing values are defined", {
   x <- labelled_spss(1:10, c(Good = 1, Bad = 8), na_values = c(9, 10))
-  expect_error(`val_labels<-`(x, NULL))
+  val_labels(x) <- NULL
+  expect_null(val_labels(x))
 
   x <- labelled_spss(1:10, c(Good = 1), na_range = c(9, 20))
-  expect_error(`val_label<-`(x, 1, NULL))
+  val_labels(x) <- NULL
+  expect_null(val_labels(x))
 })
 
 # remove_labels --------------------------------------------------------------
@@ -94,6 +143,7 @@ test_that("strict option of to_factor works correctly", {
   v <- labelled(c(1, 1, 2, 3), labels = c(No = 1, Yes = 2))
   expect_s3_class(to_factor(v, strict = FALSE), "factor")
   expect_s3_class(to_factor(v, strict = TRUE), "haven_labelled")
+  expect_equal(class(to_factor(v, strict = TRUE, unclass = TRUE)), "numeric")
 })
 
 test_that("to_factor works on data.frame", {
@@ -193,9 +243,14 @@ test_that("set_variable_labels updates variable labels", {
 
 # missing values -------------------------------------------------------------------
 
-test_that("it is not possible to define missing values if no value labels were defined", {
-  expect_error(`na_values<-`(1:3, 9))
-  expect_error(`na_range<-`(1:3, c(9, Inf)))
+test_that("it is possible to define missing values if no value labels were defined", {
+  x <- c(1, 2, 2, 9)
+  na_values(x) <- 9
+  expect_equal(na_values(x), 9)
+
+  x <- c(1, 2, 2, 9)
+  na_range(x) <- 9:10
+  expect_equal(na_range(x), 9:10)
 })
 
 # recode (dplyr) -------------------------------------------------------------------
